/***************************************************************************
 *   Copyright (C) 2003 by Martin Galpin                                   *
 *   martin@nemohackers.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "kuake.h"

#include <qwidget.h>
#include <qwidgetstack.h>
#include <qevent.h>

#include <kwin.h>
#include <kurl.h>
#include <klibloader.h>
#include <klocale.h>
#include <kaboutapplication.h>
#include <kpopupmenu.h>
#include <klocale.h>
#include <kshortcut.h>
#include <kaction.h>
#include <kstdaccel.h>
#include <kmessagebox.h>
#include <kstartupinfo.h>
#include <kconfig.h>
#include <kglobalaccel.h>
#include <kkeydialog.h>

#include "kuakepushbutton.h"
#include "kuakecustomsize.h"


// Main constructor for this class. Embeds the KPart konsole into a KParts::MainWindow and
// initialises any varibles needed for the class.
Kuake::Kuake()
    : KParts::MainWindow(0, "Kuake", WStyle_Customize|WStyle_NoBorder), 
	 DCOPObject("KuakeIface")
{
  QWidget* tmp = QApplication::desktop();
  this->x = tmp->width();
  this->y = tmp->height();
  setGeometry(0,0,x,10);

  stack = new QWidgetStack(this);
  stack->hide();

  KLibFactory *factory = 0;
  factory = KLibLoader::self()->factory("libkonsolepart");
  part = 0L;

  if (factory)
    {
      part = static_cast<KParts::ReadOnlyPart *>(factory->create(this,"libkonsolepart",
        "KParts::ReadOnlyPart"));
      if (part)
        {
          KGlobal::locale()->insertCatalogue("konsole");
          part->widget()->show();
          part->setSelectable(true);
          stack->addWidget(part->widget());
          connect(part, SIGNAL(destroyed()), this, SLOT(slotDestroyed()));
        }
    }

  pbResize = new KuakePushButton(this, "resize");
  
  ctx = new KPopupMenu(this);
  actionCollection = new KActionCollection(this, "action");
  //KAction* actPrefs = KStdAction::preferences(this, SLOT(slotPrefs()), actionCollection);
  
  	ctxSize = new KPopupMenu(ctx);
	ctxSizeTinyID = ctxSize->insertItem("Tiny", this, SLOT(slotSizeTiny()));   // 150
	ctxSizeSmallID = ctxSize->insertItem("Small", this, SLOT(slotSizeSmall()));  // 200
	ctxSizeNormalID = ctxSize->insertItem("Normal", this, SLOT(slotSizeNormal())); // 250
	ctxSizeLargeID = ctxSize->insertItem("Large", this, SLOT(slotSizeLarge()));  // 350
	ctxSizeCustomID = ctxSize->insertItem("Custom...", this, SLOT(slotSizeCustom())); // ?
	
	ctxAlignment = new KPopupMenu(ctx);
	ctxAlignmentTopID = ctxAlignment->insertItem ("Top", this, SLOT(slotAlignmentTop()));
	ctxAlignmentBottomID = ctxAlignment->insertItem ("Bottom", this, SLOT(slotAlignmentBottom()));
	ctxAlignmentLeftID = ctxAlignment->insertItem ("Left", this, SLOT(slotAlignmentLeft()));
	ctxAlignmentRightID = ctxAlignment->insertItem ("Right", this, SLOT(slotAlignmentRight()));
	
  ctx->insertItem("Si&ze", ctxSize);
  ctx->insertItem("&Alignment", ctxAlignment);
  ctx->insertItem("Toggle key...", this, SLOT(slotToggleKey()));
  //ctxEnableTabsID = ctx->insertItem("&Enable Tabs");
  KAction* actAbout = KStdAction::aboutApp(this, SLOT(slotAbout()), actionCollection);
  KAction* actQuit = KStdAction::quit(this, SLOT(slotQuit()), actionCollection);
  ctx->insertSeparator();
  actAbout->plug(ctx);
  actQuit->plug(ctx);
  
  KConfig conf("kuakerc");
  compact = conf.readBoolEntry("state", true);
  
  globalAccel = new KGlobalAccel(this);
  globalAccel->insert("ToggleState", i18n("Toggle State"), i18n("Toggles the state of Kuake"), 0, 0,
  					this, SLOT(slotChangeState()));
  globalAccel->readSettings(&conf);
  globalAccel->updateConnections();
    
  // Correct calls to the window manager 
  KWin::setActiveWindow (winId());    
  KWin::setState (winId(), NET::StaysOnTop | NET::Sticky | NET::SkipTaskbar);  
 
  readConfig();    
  change();
  //changeState();
  resizeKuake();
  emit slotChangeState();
  
  show();
    
  KStartupInfoId id;
  id.initId(kapp->startupId());
  KStartupInfo::sendFinish(id);
}
// --------------------------------------------------------------- //

Kuake::~Kuake()
{
	// Qt will handle the deletion of child widgets.
}
// --------------------------------------------------------------- //

// Called by the DCOP client in order to change the state of Kuake.
void
Kuake::setKuakeState(int i)
{
	emit slotChangeState();
	
}
// --------------------------------------------------------------- //

// Simply calls the interface changing functions.
void 
Kuake::slotChangeState()
{
	resizeKuake();
	changeState();
}
// --------------------------------------------------------------- //

// Changes the state of Kuake. When COMPACT is true the widget is minimal
// and needs to show() the STACK child and make this window active.
void
Kuake::changeState()
{
	if (compact)
		{
		   stack->show();
			raise();
			setActiveWindow();
			compact = false;
			activeWindow = false;
		}
	else 
		{
			stack->hide();
			compact = true;
			activeWindow = true;
		}
}
// --------------------------------------------------------------- //

// This method resizes Kuake to the ORIENTATION and SIZE varibles set by
// the user. It will resize Kuake and change it's state.
void
Kuake::resizeKuake()
{
  if (compact)            // Enlarge
    {
	 	switch (alignment)
			{
			case _ALIGN_TOP:
      		setGeometry(0,0,x,size);
      		pbResize->setGeometry(0,size-10,x,10);
      		stack->setGeometry(0,0,x,size-10);
				KWin::setStrut(winId(), 0, 0, 10, 0);
			break;
			case _ALIGN_BOTTOM:
				setGeometry(0,(y-size),x,size);
      		pbResize->setGeometry(0,0,x,10);
      		stack->setGeometry(0,10,x,(size-10));
				KWin::setStrut(winId(),0,0,0,10);
			break;
			case _ALIGN_LEFT:
				setGeometry(0,0,size,y);
				pbResize->setGeometry((size-10),0,10,y);
				stack->setGeometry(0,0,size-10,y);
				KWin::setStrut(winId(),10,0,0,0);
			break;
			case _ALIGN_RIGHT:
				setGeometry ((x-size),0,size,y);
				pbResize->setGeometry (0,0,10,y);
				stack->setGeometry(10,0,size,y);
				KWin::setStrut(winId(),0,10,0,0);
			break;
			}
    }
  else                    // Compact
    {
	 	switch (alignment)
			{
			case _ALIGN_TOP:
      		setGeometry(0,0,x,10);
      		pbResize->setGeometry(0,0,x,10);
				KWin::setStrut(winId(), 0, 0, 10, 0);
			break;
			case _ALIGN_BOTTOM:
				setGeometry(0,(y-10),x,10);
      		pbResize->setGeometry(0,0,x,10);
				KWin::setStrut(winId(), 0, 0, 0, 10);
			break;
			case _ALIGN_LEFT:
				setGeometry(0,0,10,y);
      		pbResize->setGeometry(0,0,10,y);
				KWin::setStrut(winId(), 10, 0, 0, 0);
			break;
			case _ALIGN_RIGHT:
				setGeometry((x-10),0,10,y);
      		pbResize->setGeometry(0,0,10,y);
				KWin::setStrut(winId(), 0, 10, 0, 0);
			break;
			}			
    }
}
// --------------------------------------------------------------- //

// This method resizes Kuake to the ORIENTATION and SIZE varibles set by
// the user. It will resize Kuake without changing the state, thus should
// be used when changing is "on the fly".
void
Kuake::resizeKeepingState()
{
  if (!compact)
    {
	 	switch (alignment)
			{
			case _ALIGN_TOP:
      		setGeometry(0,0,x,size);
      		pbResize->setGeometry(0,size-10,x,10);
      		stack->setGeometry(0,0,x,size-10);
				KWin::setStrut(winId(), 0, 0, 10, 0);
			break;
			case _ALIGN_BOTTOM:
				setGeometry(0,(y-size),x,size);
      		pbResize->setGeometry(0,0,x,10);
      		stack->setGeometry(0,10,x,(size-10));
				KWin::setStrut(winId(),0,0,0,10);
			break;
			case _ALIGN_LEFT:
				setGeometry(0,0,size,y);
				pbResize->setGeometry((size-10),0,10,y);
				stack->setGeometry(0,0,size-10,y);
				KWin::setStrut(winId(),10,0,0,0);
			break;
			case _ALIGN_RIGHT:
				setGeometry ((x-size),0,size,y);
				pbResize->setGeometry (0,0,10,y);
				stack->setGeometry(10,0,size,y);
				KWin::setStrut(winId(),0,10,0,0);
			break;
			}
    }
  else               
    {
	 	switch (alignment)
			{
			case _ALIGN_TOP:
      		setGeometry(0,0,x,10);
      		pbResize->setGeometry(0,0,x,10);
				KWin::setStrut(winId(), 0, 0, 10, 0);
			break;
			case _ALIGN_BOTTOM:
				setGeometry(0,(y-10),x,10);
      		pbResize->setGeometry(0,0,x,10);
				KWin::setStrut(winId(), 0, 0, 0, 10);
			break;
			case _ALIGN_LEFT:
				setGeometry(0,0,10,y);
      		pbResize->setGeometry(0,0,10,y);
				KWin::setStrut(winId(), 10, 0, 0, 0);
			break;
			case _ALIGN_RIGHT:
				setGeometry((x-10),0,10,y);
      		pbResize->setGeometry(0,0,10,y);
				KWin::setStrut(winId(), 0, 10, 0, 0);
			break;
			}			
    }
}
// --------------------------------------------------------------- //

// This method is an overloaded function which changes the state of Kuake when
// the user selects a different window.
void
Kuake::windowActivationChange(bool oldActive)
{
  if (!isActiveWindow() && !activeWindow)
    {
	 	activeWindow = true;
      emit slotChangeState();
    }
}
// --------------------------------------------------------------- //

// This method reads the saved settings of alignment and size. It should only be
// called when the instance of this class is created as all varibles are stored as
// members to this class.
void
Kuake::readConfig()
{
	KConfig conf ("kuakerc");
	
	size = conf.readNumEntry ("size", _SIZE_NORMAL);
	ctxSize->setItemChecked (ctxSizeCheckedID, false);
	switch (size)
		{
		case _SIZE_TINY:
			ctxSize->setItemChecked (ctxSizeTinyID, true);
			ctxSizeCheckedID = ctxSizeTinyID;
		break;
		case _SIZE_SMALL:
			ctxSize->setItemChecked (ctxSizeSmallID, true);
			ctxSizeCheckedID = ctxSizeSmallID;
		break;
		case _SIZE_NORMAL:
			ctxSize->setItemChecked (ctxSizeNormalID, true);
			ctxSizeCheckedID = ctxSizeNormalID;
		break;
		case _SIZE_LARGE:
			ctxSize->setItemChecked (ctxSizeLargeID, true);
			ctxSizeCheckedID = ctxSizeLargeID;
		break;
		default:
			ctxSize->setItemChecked (ctxSizeCustomID, true);
			ctxSizeCheckedID = ctxSizeCustomID;
		break;
		}
	
	alignment = conf.readNumEntry ("alignment", 0);
	ctxAlignment->setItemChecked (ctxAlignmentCheckedID, false);
	switch (alignment)
	 {
	 	case _ALIGN_TOP:
			ctxAlignment->setItemChecked (ctxAlignmentTopID, true);
			ctxAlignmentCheckedID = ctxAlignmentTopID;
		break;
	 	case _ALIGN_BOTTOM:
			ctxAlignment->setItemChecked (ctxAlignmentBottomID, true);
			ctxAlignmentCheckedID = ctxAlignmentBottomID;
		break;
	 	case _ALIGN_LEFT:
			ctxAlignment->setItemChecked (ctxAlignmentLeftID, true);
			ctxAlignmentCheckedID = ctxAlignmentLeftID;
		break;
	 	case _ALIGN_RIGHT:
			ctxAlignment->setItemChecked (ctxAlignmentRightID, true);
			ctxAlignmentCheckedID = ctxAlignmentRightID;
		break;
	}
}
// --------------------------------------------------------------- //

// This method syncronises the size and alignment settings with the
// checkboxes in CTX's menu item.
void
Kuake::change()
{
	// The konsole is down, so we must change it's size.
	ctxSize->setItemChecked (ctxSizeCheckedID, false);
	switch (size)
		{
		case _SIZE_TINY:
			ctxSize->setItemChecked (ctxSizeTinyID, true);
			ctxSizeCheckedID = ctxSizeTinyID;
		break;
		case _SIZE_SMALL:
			ctxSize->setItemChecked (ctxSizeSmallID, true);
			ctxSizeCheckedID = ctxSizeSmallID;
		break;
		case _SIZE_NORMAL:
			ctxSize->setItemChecked (ctxSizeNormalID, true);
			ctxSizeCheckedID = ctxSizeNormalID;
		break;
		case _SIZE_LARGE:
			ctxSize->setItemChecked (ctxSizeLargeID, true);
			ctxSizeCheckedID = ctxSizeLargeID;
		break;
		default:
			ctxSize->setItemChecked (ctxSizeCustomID, true);
			ctxSizeCheckedID = ctxSizeCustomID;
		break;
		}
	
	ctxAlignment->setItemChecked (ctxAlignmentCheckedID, false);
	switch (alignment)
	 {
	 	case _ALIGN_TOP:
			ctxAlignment->setItemChecked (ctxAlignmentTopID, true);
			ctxAlignmentCheckedID = ctxAlignmentTopID;
		break;
	 	case _ALIGN_BOTTOM:
			ctxAlignment->setItemChecked (ctxAlignmentBottomID, true);
			ctxAlignmentCheckedID = ctxAlignmentBottomID;
		break;
	 	case _ALIGN_LEFT:
			ctxAlignment->setItemChecked (ctxAlignmentLeftID, true);
			ctxAlignmentCheckedID = ctxAlignmentLeftID;
		break;
	 	case _ALIGN_RIGHT:
			ctxAlignment->setItemChecked (ctxAlignmentRightID, true);
			ctxAlignmentCheckedID = ctxAlignmentRightID;
		break;
	}	
}
// --------------------------------------------------------------- //

// This slot is emitted when the user selects "Quit" from the konsoles context menu.
void
Kuake::slotDestroyed ()
{
	emit slotQuit();
}
// --------------------------------------------------------------- //

// Self explainitry.
void
Kuake::slotAbout()
{
	KAboutApplication* about = new KAboutApplication(this);
	about->show();
}
// --------------------------------------------------------------- //

// Saves the state of Kuake so that the next time we start up, it's restored.
void
Kuake::slotQuit()
{
	KConfig conf("kuakerc");
	if (compact)
		{
			conf.writeEntry("state", "false");
		}
	else 
		{
			conf.writeEntry("state", "true");
		}
	qApp->quit();
}
// --------------------------------------------------------------- //

// The following methods are all emitted when the appropriate item
// is selected in the context menu.

void
Kuake::slotSizeTiny()
{
  KConfig conf ("kuakerc");
  conf.writeEntry("size",_SIZE_TINY);
  size = _SIZE_TINY;
  change();
  resizeKeepingState();
}
// --------------------------------------------------------------- //

void 
Kuake::slotSizeSmall()
{
  KConfig conf ("kuakerc");
  conf.writeEntry("size",_SIZE_SMALL);
  size = _SIZE_SMALL;
  change();
  resizeKeepingState();
}
// --------------------------------------------------------------- //

void 
Kuake::slotSizeNormal()
{
  KConfig conf ("kuakerc");
  conf.writeEntry ("size",_SIZE_NORMAL);
  size = _SIZE_NORMAL;
  resizeKeepingState();
}
// --------------------------------------------------------------- //

void 
Kuake::slotSizeLarge()
{
  KConfig conf ("kuakerc");
  conf.writeEntry ("size",_SIZE_LARGE);
  size = _SIZE_LARGE;
  resizeKeepingState();
}
// --------------------------------------------------------------- //

void 
Kuake::slotSizeCustom()
{
	KuakeCustomSize* cus = new KuakeCustomSize(this, 0, &size);	
}
// --------------------------------------------------------------- //

void
Kuake::slotCustomSizeOkClicked()
{
	change();
}
// --------------------------------------------------------------- //

void
Kuake::slotAlignmentTop()
{
  KConfig conf ("kuakerc");
  conf.writeEntry ("alignment",_ALIGN_TOP);
  alignment = _ALIGN_TOP;
  change();
  resizeKeepingState();
}
// --------------------------------------------------------------- //

void
Kuake::slotAlignmentBottom()
{
  KConfig conf ("kuakerc");
  conf.writeEntry ("alignment",_ALIGN_BOTTOM);
  alignment = _ALIGN_BOTTOM;
  change();
  resizeKeepingState();
}
// --------------------------------------------------------------- //

void
Kuake::slotAlignmentLeft()
{
  KConfig conf ("kuakerc");
  conf.writeEntry ("alignment",_ALIGN_LEFT);
  alignment = _ALIGN_LEFT;
  change();
  resizeKeepingState();
}
// --------------------------------------------------------------- //

void
Kuake::slotAlignmentRight()
{
  KConfig conf ("kuakerc");
  conf.writeEntry ("alignment",_ALIGN_RIGHT);
  alignment = _ALIGN_RIGHT;
  change();
  resizeKeepingState();
}
// --------------------------------------------------------------- //

// Show the KKey dialog for configuring the toggle key
void
Kuake::slotToggleKey()
{
  KKeyDialog::configure(globalAccel);
  KConfig conf ("kuakerc");
  globalAccel->updateConnections();
  globalAccel->writeSettings(&conf);  
}

#include "kuake.moc"
