/***************************************************************************
 *   Copyright (C) 2003 by Martin Galpin                                   *
 *   martin@nemohackers.org                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
 
 /* 
	Kuake Main Window. Embeds the KPart konsole and handles all window 
	operations. 
	For documentated functions see definition - kuake.cpp
 */

#ifndef _KUAKE_H_
#define _KUAKE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kparts/mainwindow.h>
#include "kuakeiface.h"

#define _ALIGN_TOP 0
#define _ALIGN_BOTTOM 1
#define _ALIGN_LEFT 2
#define _ALIGN_RIGHT 3
#define _SIZE_TINY 150
#define _SIZE_SMALL 200
#define _SIZE_NORMAL 250
#define _SIZE_LARGE 350


class QWidgetStack;
class QString;
class KuakePushButton;
class KActionCollection;
class KLibFactory;
class KPopupMenu;
class KGlobalAccel;

class Kuake : public KParts::MainWindow, virtual public KuakeIface
{
    Q_OBJECT
public:
    Kuake();
    virtual ~Kuake();
	 
	 virtual void setKuakeState(int i);
	 
	 KPopupMenu* ctx;
protected:
    virtual void windowActivationChange(bool oldActive);
	 void readConfig();
	 void change();
	 void resizeKuake();
	 void resizeKeepingState();
	 void changeState();

public slots:
    void slotChangeState();
    void notifySize(int,int) {};
    void changeColumns(int) {};
    void changeTitle(int,const QString&) {};    
    void slotDestroyed();
	 
    void slotAbout();
	 void slotQuit();
	 
	 void slotSizeTiny();
	 void slotSizeSmall();
	 void slotSizeNormal();
	 void slotSizeLarge();
	 void slotSizeCustom();
	 
	 void slotCustomSizeOkClicked();
	 
	 void slotAlignmentTop();
	 void slotAlignmentBottom();
	 void slotAlignmentRight();
	 void slotAlignmentLeft();
	 
	 void slotToggleKey();

private:
    unsigned int x;
    unsigned int y;
	 unsigned int size;
	 unsigned int alignment;
    bool compact;
	 bool activeWindow;

	 unsigned int ctxSizeTinyID;	 
	 unsigned int ctxSizeSmallID;
	 unsigned int ctxSizeNormalID;
	 unsigned int ctxSizeLargeID;
	 unsigned int ctxSizeCustomID;
	 unsigned int ctxSizeCheckedID;
	 unsigned int ctxAlignmentTopID;
	 unsigned int ctxAlignmentBottomID;
	 unsigned int ctxAlignmentRightID;
	 unsigned int ctxAlignmentLeftID;
	 unsigned int ctxAlignmentCheckedID;
	 //unsigned int ctxEnableTabsID;
	 
                                
    KParts::ReadOnlyPart* part;

    QWidgetStack* stack;
    KuakePushButton* pbResize;
	 
	 KActionCollection* actionCollection;
	 KGlobalAccel* globalAccel;
	 
	 KPopupMenu* ctxSize;
	 KPopupMenu* ctxAlignment;
};

#endif // _KUAKE_H_
